--Return the direction (in degrees) to a point.
function point_direction(x1,y1,x2,y2)
	return math.deg(math.atan2((y1-y2),(x1-x2)));
end

--Return the distance to a point.
function point_distance(x1,y1,x2,y2)
        return (math.sqrt(math.pow(x2-x1,2)+math.pow(y2-y1,2)));
end

--Return the x coordinate of a point dist pixels away in direction ang.
function lengthdir_x(dist,ang)
	return dist * math.cos(ang*math.pi/180);
end

--Return the y coordinate of a point dist pixels away in direction ang.
function lengthdir_y(dist,ang)
	return dist * math.sin(ang*math.pi/180);
end

function GravitateMovableObjectB(object,pointx,pointy,grav,rotate,maxrange)
	local dir = point_direction(object.Pos.X,object.Pos.Y,pointx,pointy);

	if rotate == 1 then
		object.RotAngle = -dir;
	end

	local mod = (-point_distance(object.Pos.X,object.Pos.Y,pointx,pointy)/maxrange)+1;
	if mod < 0 then
		mod = 0;
	end

	if object.PinStrength == 0 then
		object.Vel = Vector(object.Vel.X - lengthdir_x(grav*mod,dir),object.Vel.Y - lengthdir_y(grav*mod,dir));
	end
end

function Create(self)
	--How long this has been going on for.
	self.scriptTimer = Timer();

	--Times between small burning rocks.
	self.rocktimer = Timer();

	--How long to wait before spawning a small burning rock.
	self.rocktime = 500;

	--Times between boulders.
	self.bouldertimer = Timer();

	--How long to wait before spawning a boulder.
	self.bouldertime = 1000;

	--Times between floating rocks.
	self.uprocktimer = Timer();

	--How long to wait before spawning a floating rock.
	self.uprocktime = 500;

	--Pin the ship in the sky.
	self.PinStrength = 10000;

	--List of letters.
	self.letters = { "A", "B", "C", "D", "E" };

	--How strong the wind should be.
	self.wind = (math.random() - 0.5) / 5;

	--Whether the big crack as happened yet.
	self.hascracked = false;

	--Whether the explosion has happened yet.
	self.hasexploded = false;

	--How much force to apply upwards to avert gravity.
	self.antigrav = (SceneMan.Scene.GlocalAcc.Y * TimerMan.DeltaTimeSecs) + 0.01;

	--Radius of the black hole.
	self.holesize = 250;

	--Pull strength of the black hole.
	self.holestrength = 0.5;

	--Where to land.
	self.landspot = SceneMan:MovePointToGround(self.Pos,0,0);

	--In or out black hole next?
	self.holein = true;

	--The black hole.
	self.bhole = nil;

	--Whether the chosen actor is vincible or not.
	self.vincible = true;

	--Play the song.
	AudioMan:StopMusic();
	AudioMan:PlayMusic("Apple Corps.rte/Explosives/Karoshi Rain/anti-sad song.ogg",0,100);

	--The music music volume before changes.
	self.oldvol = AudioMan.MusicVolume;
end

function Update(self)
	--Get the actor info.
	if self.ctype == nil then
		--Find the actor.
		local curdist = 50;
		local found = nil;
		for actor in MovableMan.Actors do
			local avgx = actor.Pos.X - self.Pos.X;
			local avgy = actor.Pos.Y - self.Pos.Y;
			local dist = math.sqrt(avgx ^ 2 + avgy ^ 2);
			if dist < curdist then
				curdist = dist;
				found = actor;
			end
		end

		--If no actor was found, try again next frame.  Otherwise, continue.
		if found ~= nil then
			found.ToDelete = true;
			ActivityMan:GetActivity():ReportDeath(found.Team,-1);
			self.ctype = found.ClassName;
			self.cname = found.PresetName;
			self.cteam = found.Team;
		end
	end

	--Rain.
	if self.scriptTimer:IsPastRealMS(60000) == false then
		local drop = nil;
		drop = CreateAEmitter("Splashing Rain Drop");
		drop.Pos.X = math.random(SceneMan.Scene.Width);
		drop.Vel.Y = 45;
		MovableMan:AddParticle(drop);
		if self.scriptTimer:IsPastRealMS(5000) then
			for i=1,math.random(5) do
				drop = CreateAEmitter("Splashing Rain Drop");
				drop.Pos.X = math.random(SceneMan.Scene.Width);
				drop.Vel.Y = 45;
				MovableMan:AddParticle(drop);
			end
		end
	end

	--Wind.
	if self.scriptTimer:IsPastRealMS(34000) and self.scriptTimer:IsPastRealMS(140000) == false then
		for actor in MovableMan.Actors do
			if actor.PinStrength == 0 then
				actor.Vel.X = actor.Vel.X + self.wind;
			end
		end
		for item in MovableMan.Items do
			if item.PinStrength == 0 then
				item.Vel.X = item.Vel.X + self.wind;
			end
		end
		for particle in MovableMan.Particles do
			if particle.PinStrength == 0 then
				particle.Vel.X = particle.Vel.X + self.wind;
			end
		end
	end

	--Small lightning.
	if self.scriptTimer:IsPastRealMS(34000) and self.scriptTimer:IsPastRealMS(60000) == false then
		if math.random() > 0.99 then
			local flash = CreateAEmitter("Lightning Flash " .. math.random(3) .. " With Sound");
			flash.Pos = SceneMan:MovePointToGround(Vector(math.random(SceneMan.Scene.Width),0),0,0);
			MovableMan:AddParticle(flash);
		end
	end

	--Big crack.
	if self.hascracked == false and self.scriptTimer:IsPastRealMS(60000) then
		local crack = CreateAEmitter("Lightning With Sound");
		crack.Pos = SceneMan:MovePointToGround(self.Pos,0,0);
		MovableMan:AddParticle(crack);
		self.hascracked = true;
	end

	--Small burning boulders.
	if self.scriptTimer:IsPastRealMS(60000) and self.scriptTimer:IsPastRealMS(116000) == false then
		local rock = nil;
		if self.rocktimer:IsPastRealMS(self.rocktime) then
			rock = CreateAEmitter("Flaming Boulder Chunk " .. self.letters[math.random(3)]);
			rock.Pos.X = math.random(SceneMan.Scene.Width);
			MovableMan:AddParticle(rock);
			self.rocktimer:Reset();
		end
	end

	--Large exploding rocks.
	if self.scriptTimer:IsPastRealMS(116000) and self.scriptTimer:IsPastRealMS(140000) == false then
		local boulder = nil;
		if self.bouldertimer:IsPastRealMS(self.bouldertime) then
			boulder = CreateAEmitter("Flaming Boulder " .. self.letters[math.random(3)]);
			boulder.Pos.X = math.random(SceneMan.Scene.Width);
			MovableMan:AddParticle(boulder);
			self.bouldertimer:Reset();
		end
	end

	--Gravity displacement.
	if self.scriptTimer:IsPastRealMS(140000) and self.scriptTimer:IsPastRealMS(163000) == false then
		for actor in MovableMan.Actors do
			actor.Vel.Y = actor.Vel.Y - self.antigrav;
		end
		for item in MovableMan.Items do
			item.Vel.Y= item.Vel.Y - self.antigrav;
		end
		for particle in MovableMan.Particles do
			particle.Vel.Y = particle.Vel.Y - self.antigrav;
		end
	end

	--Black hole.
	if self.scriptTimer:IsPastRealMS(163000) and self.scriptTimer:IsPastRealMS(203000) == false then
		for actor in MovableMan.Actors do
			GravitateMovableObjectB(actor,self.landspot.X,self.landspot.Y - 250,self.holestrength,0,self.holesize);
			actor.Vel.Y = actor.Vel.Y - self.antigrav;
		end
		for item in MovableMan.Items do
			GravitateMovableObjectB(item,self.landspot.X,self.landspot.Y - 250,self.holestrength,0,self.holesize);
			item.Vel.Y= item.Vel.Y - self.antigrav;
		end
		for particle in MovableMan.Particles do
			if particle.ClassName == "MOSRotating" or particle.ClassName == "Attachable" or particle.ClassName == "AEmitter" then
				GravitateMovableObjectB(particle,self.landspot.X,self.landspot.Y - 250,self.holestrength,0,self.holesize);
			end
			particle.Vel.Y = particle.Vel.Y - self.antigrav;
		end
		self.holesize = self.holesize + 5;
		self.holestrength = self.holestrength + 0.1;
		if MovableMan:IsParticle(self.bhole) == true then
			if self.bhole.Age > 998 then
				if self.holein == true then
					self.bhole = CreateMOPixel("Coming Black Hole Glow In");
					self.bhole.Pos = self.landspot - Vector(0,250);
					MovableMan:AddParticle(self.bhole);
					self.holein = false;
				else
					self.bhole = CreateMOPixel("Coming Black Hole Glow Out");
					self.bhole.Pos = self.landspot - Vector(0,250);
					MovableMan:AddParticle(self.bhole);
					self.holein = true;
				end
			end
		else
			if self.holein == true then
				self.bhole = CreateMOPixel("Coming Black Hole Glow In");
				self.bhole.Pos = self.landspot - Vector(0,250);
				MovableMan:AddParticle(self.bhole);
				self.holein = false;
			else
				self.bhole = CreateMOPixel("Coming Black Hole Glow Out");
				self.bhole.Pos = self.landspot - Vector(0,250);
				MovableMan:AddParticle(self.bhole);
				self.holein = true;
			end
		end
	end

	--Rocks falling up.
	if self.scriptTimer:IsPastRealMS(180000) and self.scriptTimer:IsPastRealMS(203000) == false then
		local uprock = nil;
		if self.uprocktimer:IsPastRealMS(self.uprocktime) then
			uprock = CreateAEmitter("Boulder Bit " .. self.letters[math.random(5)]);
			uprock.Pos = SceneMan:MovePointToGround(Vector(math.random(SceneMan.Scene.Width),0),0,0);
			uprock.Vel.Y = -15;
			MovableMan:AddParticle(uprock);
			self.uprocktimer:Reset();
		end
	end

	--Explosion.
	if self.scriptTimer:IsPastRealMS(203000) and self.hasexploded == false then
		if MovableMan:IsParticle(self.bhole) == true then
			self.bhole.ToDelete = true;
		end
		local explosion = CreateAEmitter("Coming Explosion");
		explosion.Pos = self.landspot - Vector(0,250);
		MovableMan:AddParticle(explosion);
		self.hasexploded = true;

		if self.ctype == "AHuman" then
			self.chosen = CreateAHuman(self.cname);
		elseif self.ctype == "Actor" then
			self.chosen = CreateActor(self.cname);
		elseif self.ctype == "ACrab" then
			self.chosen = CreateACrab(self.cname);
		end

		--Set the actor's team.
		self.chosen.Team = self.cteam;

		--Add the actor.
		MovableMan:AddActor(self.chosen);
	end

	--Move the actor down.
	if MovableMan:IsActor(self.chosen) and self.scriptTimer:IsPastRealMS(203000) then
		if self.vincible == true then
			--Make the actor invincible.  Store the parts in a list so we can set them back afterward.
			self.partslist = {};

			for i=1,MovableMan:GetMOIDCount()-1 do
				if MovableMan:GetMOFromID(i).RootID == self.chosen.ID then
					self.partslist[#self.partslist + 1] = MovableMan:GetMOFromID(i);
					self.partslist[#self.partslist].GetsHitByMOs = false;
				end
			end

			--If we actually found any, then we say the actor is invincible.  If not, try again next turn.
			if #self.partslist > 0 then
				self.vincible = false;
			end
		end

		--Used for measuring a few gradual things.
		local closeness = 1 - (self.chosen.Pos.Y / self.landspot.Y);

		--Move the actor gradually down until it reaches the desired position.
		self.chosen.Vel = Vector(0,0);
		self.chosen.Pos.X = self.Pos.X;
		self.chosen.AngularVel = 0;
		self.chosen.RotAngle = 0;

		local downam = closeness * 5;
		if downam < 1 then
			downam = 1;
		end
		self.chosen.Pos.Y = self.chosen.Pos.Y + downam;

		--Make the actor glow.
		self.chosen:FlashWhite(1);

		--Make the actor face forward.
		self.chosen:SetAimAngle(0);

		--Shut off the AI so it doesn't try to escape.
		self.chosen:SetControllerMode(Controller.CIM_DISABLED,-1);

		--Make it rain.
		if math.random() < closeness then
			local drop = nil;
			drop = CreateAEmitter("Splashing Rain Drop");
			drop.Pos.X = math.random(SceneMan.Scene.Width);
			drop.Vel.Y = 45;
			MovableMan:AddParticle(drop);
		end

		--Fade out music gradually.
		local vol = closeness * 2;
		if vol > 1 then
			vol = 1;
		end
		AudioMan.MusicVolume = vol;

		--End this all if the chosen actor has arrived at its position.
		if self.chosen.Pos.Y > self.landspot.Y - 15 and self.chosen.Pos.Y < self.landspot.Y + 15 then
			self.ToDelete = true;
			self.chosen:SetControllerMode(Controller.CIM_AI,-1);

			--Make the actor vincible again.
			for i=1,#self.partslist do
				self.partslist[i].GetsHitByMOs = true;
			end

			--Stop music and reset settings.
			AudioMan:StopMusic();
			AudioMan.MusicVolume = self.oldvol;
			AudioMan:ClearMusicQueue();

			self.chosen.PinStrength = 0;
		end
	end
end